///File : findFile.cpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "findFile.hpp"
//------------------------------------------------------------------------------
wxString findFile(const wxString &str)
{
    wxString testPath;

    testPath = str;
    if(wxFileExists(testPath) || wxDirExists(testPath)) // Si c'est dans le dossier ./
    {
        return testPath;
    }

    testPath = _T("../") + str;
    if(wxFileExists(testPath) || wxDirExists(testPath)) // Si c'est dans le dossier ../
    {
        return testPath;
    }

    #if defined(__UNIX__)
    testPath = wxString(wxStandardPaths().GetInstallPrefix() + _T("/share/") + _T(PACKAGE) + _T("/") + str);
    if(wxFileExists(testPath) || wxDirExists(testPath)) // Si c'est dans le dossier /usr/.../share/nomprog
    {
        return testPath;
    }
    #endif // defined(__UNIX__)

    testPath = wxString(wxStandardPaths().GetDataDir() + _T("/") + str);
    if(wxFileExists(testPath) || wxDirExists(testPath)) // Si c'est dans le dossier /usr/.../share/nomPaquet
    {
        return testPath;
    }

    // Si non
    wxMessageBox(_("File not Found : ") + str, _("Error"), wxICON_ERROR);
    return _T("");
}
//------------------------------------------------------------------------------
wxString copyToSysConfigDir(const wxString &filepath)
{
    wxString filename = wxFileName(filepath).GetFullName();
    wxString destpath = wxStandardPaths().GetConfigDir() + wxString(_T("/")) + filename;

    if(wxCopyFile(filepath, destpath, false))
    {
        return destpath;
    }
    else
    {
        return _T("");
    }
}
//------------------------------------------------------------------------------
wxString copyToUserConfigDir(const wxString &filepath)
{
    wxString filename = wxFileName(filepath).GetFullName();

    // Création du dossier ==> findFileUserDataDir() le fait.
    wxString destpath = findFileUserDataDir(_T("")) + filename;

    if(wxCopyFile(filepath, destpath, false))
    {
        #if defined(__UNIX__)
        wxExecute(_T("chmod 666 ") + destpath);
        #endif
        return destpath;
    }
    else
    {
        return _T("");
    }
}
//------------------------------------------------------------------------------
wxString findConfigFile(const wxString &str, const bool &searchInDataDir)
{
    wxString testPath;

    // 1 - Si dans dossier de tous les utilisateurs...
    testPath = wxStandardPaths().GetConfigDir() + _T("/") + str;
    if(wxFileExists(testPath))
    {
        return testPath;
    }

    // 2 - Si dans dossier de config de l'utilisateur...
    testPath = findFileUserDataDir(str);
    if(testPath != _T(""))
    {
        return testPath;
    }

    if(searchInDataDir)
    {
        // 3 - Si non ==> dossier data (fichier par défaut)
        testPath = findFile(wxString(_T("data/")) + str);
        if(testPath != _T(""))
        {
            return testPath;
        }
    }

    return(_T(""));
}
//------------------------------------------------------------------------------
wxString findFileUserDataDir(const wxString &str)
{
    const wxString userConfigPath = wxStandardPaths().GetUserConfigDir() + _T("/.") + _T(PACKAGE);

    // Création du dossier de config si il n'existe pas.
    if(!wxDirExists(userConfigPath))
    {
        wxMkdir(userConfigPath);
    }

    wxString testPath = userConfigPath + _T("/") + str;
    if(wxFileExists(testPath) || wxDirExists(testPath))
    {
        return testPath;
    }

    // Si non
    return _T("");
}
