///File : DialogProperty.cpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "DialogProperty.hpp"

//------------------------------------------------------------------------------
//----------------------------- MyFontColorPicker ------------------------------
//------------------------------------------------------------------------------
DECLARE_EVENT_TYPE( wxEVT_COMMAND_MY_FONT_COLOR_PICKER_CHANGED, -1 )

typedef void (wxEvtHandler::*wxCommandEventFunction)(wxCommandEvent&);

#define EVT_PLOT(id, fn) \
    DECLARE_EVENT_TABLE_ENTRY( wxEVT_COMMAND_MY_FONT_COLOR_PICKER_CHANGED, id, -1, \
    (wxObjectEventFunction) (wxEventFunction) (wxCommandEventFunction) \
    wxStaticCastEvent( wxCommandEventFunction, & fn ), (wxObject *) NULL ),

DEFINE_EVENT_TYPE( wxEVT_COMMAND_MY_FONT_COLOR_PICKER_CHANGED )
//------------------------------------------------------------------------------
MyFontColorPicker::MyFontColorPicker(wxWindow* parent, wxWindowID id, const wxString &label, const StructStdColorFont &data, const wxPoint& pos, const wxSize& size, long style, const wxString& name) : wxPanel(parent, id, pos, size, style, name)
{
	//SetMinSize(wxSize(100, 100));

	// Ctrls
    wxBoxSizer *MainSizer = new wxBoxSizer( wxVERTICAL );

    MainSizer->Add(new wxStaticText(this, wxID_ANY, label), 0, wxEXPAND | wxLEFT, 15);

    wxBoxSizer *CtrlSizer = new wxBoxSizer( wxHORIZONTAL );

    m_fontCtrl = new wxFontPickerCtrl(this, wxID_ANY, wxNullFont, wxDefaultPosition, wxDefaultSize, wxFNTP_FONTDESC_AS_LABEL );
    CtrlSizer->Add(m_fontCtrl, 1, wxEXPAND);

    //CtrlSizer->Add(new wxStaticText(this, wxID_ANY, _T("|")), 0, wxRIGHT | wxLEFT | wxEXPAND, 5);
    CtrlSizer->Add(new wxStaticLine(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxLI_VERTICAL ), 0, wxEXPAND | wxALL, 5);

    m_colorCtrl = new wxColourPickerCtrl(this, wxID_ANY );
    CtrlSizer->Add(m_colorCtrl, 0, wxEXPAND);

    MainSizer->Add(CtrlSizer, 1, wxEXPAND);

    SetSizer(MainSizer);

    SetStdColorFont(data);

    // Connect
    m_fontCtrl->Connect( wxEVT_COMMAND_FONTPICKER_CHANGED, wxEventHandler(MyFontColorPicker::OnChanged), NULL, this);
    m_colorCtrl->Connect( wxEVT_COMMAND_COLOURPICKER_CHANGED, wxEventHandler(MyFontColorPicker::OnChanged), NULL, this);
}
//------------------------------------------------------------------------------
void MyFontColorPicker::SetStdColorFont(const StructStdColorFont &data)
{
    StructWxColorFont wxData = stdColorFontToWxColorFont(data);

    m_fontCtrl->SetSelectedFont( wxData.wxfont );
    m_colorCtrl->SetColour( wxData.wxcolor );
}
//------------------------------------------------------------------------------
StructStdColorFont MyFontColorPicker::GetStdColorFont()
{
    StructWxColorFont wxData;

    wxData.wxfont = m_fontCtrl->GetSelectedFont();
    wxData.wxcolor = m_colorCtrl->GetColour();

    return wxColorFontToStdColorFont(wxData);
}
//------------------------------------------------------------------------------
void MyFontColorPicker::OnChanged(wxEvent &event)
{
    wxCommandEvent myEvent( wxEVT_COMMAND_MY_FONT_COLOR_PICKER_CHANGED, GetId() );
    myEvent.SetEventObject( this );
    GetEventHandler()->ProcessEvent( myEvent );
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//----------------------------- DialogProperty ---------------------------------
//------------------------------------------------------------------------------
DialogProperty::DialogProperty( wxWindow* parent, wxWindowID id, ConfigManager *cfg, ConfigManager *defaultCfg, const wxString& title, const wxPoint& pos, const wxSize& size, long style ) : wxDialog( parent, id, title, pos, size, style )
{
    // Config
    m_cfg = cfg;
    m_defaultCfg = defaultCfg;

    // Content
    wxBoxSizer *MainSizer = new wxBoxSizer(wxVERTICAL);

    m_book = new wxNotebook(this, wxID_ANY);

    MainSizer->Add(m_book, 1, wxEXPAND | wxTOP, 2);

    wxBoxSizer *BtLabelSizer = new wxBoxSizer( wxHORIZONTAL );

    m_modifiedLabel = new wxStaticText(this, wxID_ANY, _T(""));
    BtLabelSizer->Add(m_modifiedLabel, 0, wxALL | wxALIGN_CENTER, 5);
    BtLabelSizer->AddStretchSpacer();
    if(m_defaultCfg != NULL)
    {
        wxButton *defaultParamBt = new wxButton(this, wxID_ANY, _("&Default"));
        BtLabelSizer->Add(defaultParamBt, 0, wxALL | wxEXPAND | wxALIGN_CENTER_VERTICAL, 5);
        defaultParamBt->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(DialogProperty::OnDefaultCfg), NULL, this);
    }
    m_undoBt = new wxButton(this, wxID_BACKWARD );
    BtLabelSizer->Add(m_undoBt, 0, wxEXPAND | wxALL, 5);
    BtLabelSizer->Add(new wxButton(this, wxID_CANCEL), 0, wxEXPAND | wxALL, 5);
    BtLabelSizer->Add(new wxButton(this, wxID_OK), 0, wxEXPAND | wxALL, 5);

    MainSizer->Add(BtLabelSizer, 0, wxEXPAND | wxLEFT, 10);

    SetSizer(MainSizer);

    // Other
	CenterOnParent();

	// Modified
	m_isModified = false;
	m_isSubDialogModified = false;

	// Events
	Connect( wxID_BACKWARD, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(DialogProperty::OnReloadCfg));

	// Refresh
	RefreshLabels();
}
//------------------------------------------------------------------------------
wxNotebook* DialogProperty::GetNotebook()
{
    return m_book;
}
//------------------------------------------------------------------------------
void DialogProperty::OnDefaultCfg(wxCommandEvent& event)
{
    if(m_defaultCfg != NULL && wxMessageBox(_("Do you realy want to load default invoice's / estimate's parameters ?"), _("Load default parameters"), wxICON_QUESTION | wxYES_NO | wxYES_DEFAULT, this) == wxYES)
    {
        ConfigManager *normalCfgPtr = m_cfg;
        m_cfg = m_defaultCfg; // La config normale devient la config par default le temps de la mise à jour...
        for(int i = 0; i < m_paramCtrls.size(); i++)
        {
            LoadCfgForParam(i);
        }
        m_cfg = normalCfgPtr; // On remet tout comme il faut.

        m_isModified = true;
        RefreshLabels();
    }
}
//------------------------------------------------------------------------------
void DialogProperty::OnReloadCfg(wxCommandEvent& event)
{
    for(int i = 0; i < m_paramCtrls.size(); i++)
    {
        LoadCfgForParam(i);
    }
    m_isModified = false;
    RefreshLabels();
}
//------------------------------------------------------------------------------
void DialogProperty::AddParamCtrl(const int &type, wxWindow *parent, wxSizer *sizer, const std::string &param, const wxString &label, const void *specialData)
{
    wxSizer *oldSizer = sizer;
    int proportion = 0;

    if(m_paramCtrls.size() >= 1 && m_paramCtrls[m_paramCtrls.size() - 1].type == type_check_enable_next)
    {
        // On décale le ctrl vers la droite.
        sizer = new wxBoxSizer( wxVERTICAL );
    }


    wxWindow *ctrlWindow = NULL;

    if(type == type_check || type == type_check_enable_next)
    {
        wxCheckBox *ctrl = new wxCheckBox(parent, wxID_ANY, label);
        ctrl->Connect( wxEVT_COMMAND_CHECKBOX_CLICKED, wxEventHandler(DialogProperty::OnModif), NULL, this);
        if(type == type_check_enable_next)
        {
            sizer->Add(ctrl, 0, wxEXPAND | wxLEFT | wxRIGHT | wxTOP, 5 ); // On le colle au suivant
            ctrl->Connect( wxEVT_COMMAND_CHECKBOX_CLICKED, wxCommandEventHandler(DialogProperty::OnEnableNextParam), NULL, this);
        }
        else
        {
            sizer->Add(ctrl, 0, wxEXPAND | wxALL, 5 );
        }
        ctrlWindow = ctrl;
    }
    else if(type == type_singleLine || type == type_multiLine || type == type_double || type == type_double_per_cent)
    {
        int flags = 0;
        if(type == type_multiLine)
        {
            proportion = 1;
            flags = wxTE_MULTILINE;
        }

        wxBoxSizer *tempSizer = new wxBoxSizer(wxVERTICAL);
        tempSizer->Add(new wxStaticText(parent, wxID_ANY, label), 0, wxEXPAND | wxLEFT, 15);
        wxTextCtrl *ctrl;

        if(type == type_singleLine || type == type_multiLine)
        {
            ctrl = new wxTextCtrl(parent, wxID_ANY, _T(""), wxDefaultPosition, wxDefaultSize, flags);
        }
        else if(type == type_double || type == type_double_per_cent)
        {
            ctrl = new wxTextCtrl(parent, wxID_ANY, _T(""), wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
        }

        tempSizer->Add(ctrl, proportion, wxEXPAND);
        ctrlWindow = ctrl;
        ctrl->Connect( wxEVT_COMMAND_TEXT_UPDATED, wxEventHandler(DialogProperty::OnModif), NULL, this);
        sizer->Add(tempSizer, proportion, wxEXPAND | wxALL, 5 );
    }
    else if(type == type_int)
    {
        wxBoxSizer *tempSizer = new wxBoxSizer(wxHORIZONTAL);
        tempSizer->Add(new wxStaticText(parent, wxID_ANY, label), 0, wxEXPAND | wxLEFT, 5);
        wxSpinCtrl *ctrl = new wxSpinCtrl(parent, wxID_ANY);
        ctrl->SetRange(0, 2000);
        tempSizer->AddStretchSpacer();
        tempSizer->Add(ctrl, 0, wxEXPAND);
        ctrlWindow = ctrl;
        ctrl->Connect( wxEVT_COMMAND_SPINCTRL_UPDATED, wxEventHandler(DialogProperty::OnModif), NULL, this);
        sizer->Add(tempSizer, proportion, wxEXPAND | wxALL, 5 );
    }
    else if(type == type_file)
    {
        wxBoxSizer *tempSizer = new wxBoxSizer(wxVERTICAL);
        tempSizer->Add(new wxStaticText(parent, wxID_ANY, label), 0, wxEXPAND | wxLEFT, 15);
        wxFilePickerCtrl *ctrl = new wxFilePickerCtrl(parent, wxID_ANY, _T(""),_("Select a file"), _T("*.*"),  wxDefaultPosition, wxDefaultSize, wxFLP_USE_TEXTCTRL );
        tempSizer->Add(ctrl, proportion, wxEXPAND);
        ctrlWindow = ctrl;
        ctrl->Connect( wxEVT_COMMAND_FILEPICKER_CHANGED, wxEventHandler(DialogProperty::OnModif), NULL, this);
        sizer->Add(tempSizer, proportion, wxEXPAND | wxALL, 5 );
    }
    else if(type == type_dir)
    {
        wxBoxSizer *tempSizer = new wxBoxSizer(wxVERTICAL);
        tempSizer->Add(new wxStaticText(parent, wxID_ANY, label), 0, wxEXPAND | wxLEFT, 15);
        wxDirPickerCtrl *ctrl = new wxDirPickerCtrl(parent, wxID_ANY, _T(""),_("Select a folder"),  wxDefaultPosition, wxDefaultSize, wxDIRP_DEFAULT_STYLE  );
        tempSizer->Add(ctrl, proportion, wxEXPAND);
        ctrlWindow = ctrl;
        ctrl->Connect( wxEVT_COMMAND_DIRPICKER_CHANGED, wxEventHandler(DialogProperty::OnModif), NULL, this);
        sizer->Add(tempSizer, proportion, wxEXPAND | wxALL, 5 );
    }
    else if(type == type_font)
    {
        wxBoxSizer *tempSizer = new wxBoxSizer(wxVERTICAL);
        tempSizer->Add(new wxStaticText(parent, wxID_ANY, label), 0, wxEXPAND | wxLEFT, 15);
        wxFontPickerCtrl *ctrl = new wxFontPickerCtrl(parent, wxID_ANY, wxNullFont, wxDefaultPosition, wxDefaultSize, wxFNTP_FONTDESC_AS_LABEL );
        tempSizer->Add(ctrl, proportion, wxEXPAND);
        ctrlWindow = ctrl;
        ctrl->Connect( wxEVT_COMMAND_FONTPICKER_CHANGED, wxEventHandler(DialogProperty::OnModif), NULL, this);
        sizer->Add(tempSizer, proportion, wxEXPAND | wxALL, 5 );
    }
    else if(type == type_color)
    {
        wxBoxSizer *tempSizer = new wxBoxSizer(wxHORIZONTAL);
        tempSizer->Add(new wxStaticText(parent, wxID_ANY, label), 0, wxEXPAND | wxLEFT, 5);
        wxColourPickerCtrl *ctrl = new wxColourPickerCtrl(parent, wxID_ANY);
        tempSizer->AddStretchSpacer();
        tempSizer->Add(ctrl, 0, wxEXPAND);
        ctrlWindow = ctrl;
        ctrl->Connect( wxEVT_COMMAND_COLOURPICKER_CHANGED, wxEventHandler(DialogProperty::OnModif), NULL, this);
        sizer->Add(tempSizer, proportion, wxEXPAND | wxALL, 5 );
    }
    else if(type == type_choice_index)
    {
        const wxArrayString *strs = static_cast<const wxArrayString*>(specialData);

        wxBoxSizer *tempSizer = new wxBoxSizer(wxHORIZONTAL);
        tempSizer->Add(new wxStaticText(parent, wxID_ANY, label), 0, wxEXPAND | wxLEFT, 5);
        wxChoice *ctrl = new wxChoice(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, *strs);
        tempSizer->AddStretchSpacer();
        tempSizer->Add(ctrl, 0, wxEXPAND);
        ctrlWindow = ctrl;
        ctrl->Connect( wxEVT_COMMAND_CHOICE_SELECTED, wxEventHandler(DialogProperty::OnModif), NULL, this);
        sizer->Add(tempSizer, proportion, wxEXPAND | wxALL, 5 );
    }
    else if(type == type_color_font)
    {
        MyFontColorPicker *ctrl = new MyFontColorPicker(parent, wxID_ANY, label);
        ctrl->Connect( wxEVT_COMMAND_MY_FONT_COLOR_PICKER_CHANGED, wxEventHandler(DialogProperty::OnModif), NULL, this);
        sizer->Add(ctrl, 0, wxEXPAND | wxALL, 5 );
        ctrlWindow = ctrl;
    }
    else
    {
        std::string msg = "DialogProperty : unknown type \"";
        msg += nbToString(type);
        msg += "\" !";
        throw ticpp::Exception(msg.c_str());
    }

    if(m_paramCtrls.size() >= 1 && m_paramCtrls[m_paramCtrls.size() - 1].type == type_check_enable_next)
    {
        wxCheckBox *checkBox = static_cast<wxCheckBox*>(m_paramCtrls[m_paramCtrls.size() - 1].ctrl);

        // On enable le suivant en fn du checkBox
        ctrlWindow->Enable(checkBox->IsChecked());

        // On décale le ctrl vers la droite.
        oldSizer->Add(sizer, proportion, wxEXPAND | wxLEFT, 25);
    }

    StructParamCtrl tmp;
    tmp.ctrl = ctrlWindow;
    tmp.type = type;
    tmp.param = param;
    m_paramCtrls.push_back(tmp);

    LoadCfgForParam(m_paramCtrls.size() - 1); // On donne la valeur au dernier param.
}
//------------------------------------------------------------------------------
void DialogProperty::OnModif(wxEvent &event)
{
    if(event.GetEventType() == wxEVT_COMMAND_DIRPICKER_CHANGED)
    {
        for(int i = 0; i < m_paramCtrls.size(); i++)
        {
            if(m_paramCtrls[i].type == type_dir && m_paramCtrls[i].ctrl->GetId() == event.GetId())
            {
                wxFileDirPickerEvent &dirEvent = static_cast<wxFileDirPickerEvent&>(event);

                if( dirEvent.GetPath() == StrStdToWx(m_cfg->GetString(m_paramCtrls[i].param)))
                {
                    return;
                }
            }
        }
    }

    m_isModified = true;
    RefreshLabels();
}
//------------------------------------------------------------------------------
void DialogProperty::RefreshLabels()
{
    wxString label;

    if(m_isModified)
    {
        label = _("Modified");
    }
    else
    {
        label = _("Not modified");
    }

    if( m_isSubDialogModified )
    {
        label << _(", Child frame modified");
    }

    m_modifiedLabel->SetLabel(label);
    m_undoBt->Enable(m_isModified);
}
//------------------------------------------------------------------------------
void DialogProperty::OnEnableNextParam(wxCommandEvent& event)
{
    // 1 : On retrouve le check box
    int pos = -1;
    for(int i = 0; i < m_paramCtrls.size(); i++)
    {
        if(m_paramCtrls[i].ctrl->GetId() == event.GetId())
        {
            pos = i;
            break;
        }
    }

    // Test
    if( ! ((m_paramCtrls.size() - 1 >= pos + 1) /* Si existe un élément suivant */ && m_paramCtrls[pos].type == type_check_enable_next /* Si bon type */))
    {
        throw ticpp::Exception("DialogInfoSDevis : param is not a check box or is the last element !");
    }
    // On récupére le check box
    wxCheckBox *checkBox = static_cast<wxCheckBox*>(m_paramCtrls[pos].ctrl);

    // On enable le suivant en fn du checkBox
    m_paramCtrls[pos + 1].ctrl->Enable(checkBox->IsChecked());

    // Skip ==> Pour flag Modif
    event.Skip();
}
//------------------------------------------------------------------------------
bool DialogProperty::IsModified() const
{
    return m_isModified || m_isSubDialogModified;
}
//------------------------------------------------------------------------------
void DialogProperty::LoadCfgForParams()
{
    for(int i = 0; i < m_paramCtrls.size(); i++)
    {
        LoadCfgForParam(i);
    }

    m_isModified = false;
}
//------------------------------------------------------------------------------
void DialogProperty::LoadCfgForParam(const int &i)
{
    if(m_paramCtrls[i].type == type_check || m_paramCtrls[i].type == type_check_enable_next)
    {
        wxCheckBox *ctrl = static_cast<wxCheckBox*>(m_paramCtrls[i].ctrl);
        ctrl->SetValue(m_cfg->GetBool(m_paramCtrls[i].param));
    }
    else if(m_paramCtrls[i].type == type_singleLine || m_paramCtrls[i].type == type_multiLine)
    {
        wxTextCtrl *ctrl = static_cast<wxTextCtrl*>(m_paramCtrls[i].ctrl);
        ctrl->ChangeValue(StrStdToWx(m_cfg->GetString(m_paramCtrls[i].param)));
    }
    else if(m_paramCtrls[i].type == type_double || m_paramCtrls[i].type == type_double_per_cent)
    {
        wxTextCtrl *ctrl = static_cast<wxTextCtrl*>(m_paramCtrls[i].ctrl);
        double value = m_cfg->GetDouble(m_paramCtrls[i].param);

        if(m_paramCtrls[i].type == type_double_per_cent)
        {
            value *= 100.f;
        }

        ctrl->ChangeValue(wxNbToStr(value));
    }
    else if(m_paramCtrls[i].type == type_int)
    {
        wxSpinCtrl *ctrl = static_cast<wxSpinCtrl*>(m_paramCtrls[i].ctrl);
        ctrl->SetValue(m_cfg->GetInt(m_paramCtrls[i].param));
    }
    else if(m_paramCtrls[i].type == type_file)
    {
        wxFilePickerCtrl *ctrl = static_cast<wxFilePickerCtrl*>(m_paramCtrls[i].ctrl);
        ctrl->SetPath(StrStdToWx(m_cfg->GetString(m_paramCtrls[i].param)));
    }
    else if(m_paramCtrls[i].type == type_dir)
    {
        wxDirPickerCtrl *ctrl = static_cast<wxDirPickerCtrl*>(m_paramCtrls[i].ctrl);
        ctrl->SetPath(StrStdToWx(m_cfg->GetString(m_paramCtrls[i].param)));
    }
    else if(m_paramCtrls[i].type == type_font)
    {
        wxFontPickerCtrl *ctrl = static_cast<wxFontPickerCtrl*>(m_paramCtrls[i].ctrl);
        ctrl->SetSelectedFont(stdFontToWxFont(m_cfg->GetFont(m_paramCtrls[i].param)));
    }
    else if(m_paramCtrls[i].type == type_color)
    {
        wxColourPickerCtrl *ctrl = static_cast<wxColourPickerCtrl*>(m_paramCtrls[i].ctrl);
        ctrl->SetColour(stdColorToWxColor(m_cfg->GetColor(m_paramCtrls[i].param)));
    }
    else if(m_paramCtrls[i].type == type_choice_index)
    {
        wxChoice *ctrl = static_cast<wxChoice*>(m_paramCtrls[i].ctrl);
        ctrl->SetSelection(m_cfg->GetInt(m_paramCtrls[i].param));
    }
    else if(m_paramCtrls[i].type == type_color_font)
    {
        MyFontColorPicker *ctrl = static_cast<MyFontColorPicker*>(m_paramCtrls[i].ctrl);
        ctrl->SetStdColorFont(m_cfg->GetColorFont(m_paramCtrls[i].param));
    }
    else
    {
        std::string msg = "DialogProperty : unknown type \"";
        msg += nbToString(m_paramCtrls[i].type);
        msg += "\" !";
        throw ticpp::Exception(msg.c_str());
    }
}
//------------------------------------------------------------------------------
void DialogProperty::SaveCfg()
{
    if(m_isModified)
    {
        for(int i = 0; i < m_paramCtrls.size(); i++)
        {
            if(m_paramCtrls[i].type == type_check || m_paramCtrls[i].type == type_check_enable_next)
            {
                wxCheckBox *ctrl = static_cast<wxCheckBox*>(m_paramCtrls[i].ctrl);
                m_cfg->SetBool(m_paramCtrls[i].param, ctrl->IsChecked());
            }
            else if(m_paramCtrls[i].type == type_singleLine || m_paramCtrls[i].type == type_multiLine)
            {
                wxTextCtrl *ctrl = static_cast<wxTextCtrl*>(m_paramCtrls[i].ctrl);
                m_cfg->SetString(m_paramCtrls[i].param, StrWxToStd(ctrl->GetValue()));
            }
            else if(m_paramCtrls[i].type == type_double || m_paramCtrls[i].type == type_double_per_cent)
            {
                wxTextCtrl *ctrl = static_cast<wxTextCtrl*>(m_paramCtrls[i].ctrl);
                double value = wxStrToNb<double>(ctrl->GetValue());

                if(m_paramCtrls[i].type == type_double_per_cent)
                {
                    value /= 100.f;
                }

                m_cfg->SetDouble(m_paramCtrls[i].param, value);
            }
            else if(m_paramCtrls[i].type == type_int)
            {
                wxSpinCtrl *ctrl = static_cast<wxSpinCtrl*>(m_paramCtrls[i].ctrl);
                m_cfg->SetInt(m_paramCtrls[i].param, ctrl->GetValue());
            }
            else if(m_paramCtrls[i].type == type_file)
            {
                wxFilePickerCtrl *ctrl = static_cast<wxFilePickerCtrl*>(m_paramCtrls[i].ctrl);
                m_cfg->SetString(m_paramCtrls[i].param, StrWxToStd(ctrl->GetPath()));
            }
            else if(m_paramCtrls[i].type == type_dir)
            {
                wxDirPickerCtrl *ctrl = static_cast<wxDirPickerCtrl*>(m_paramCtrls[i].ctrl);
                m_cfg->SetString(m_paramCtrls[i].param, StrWxToStd(ctrl->GetPath()));
            }
            else if(m_paramCtrls[i].type == type_font)
            {
                wxFontPickerCtrl *ctrl = static_cast<wxFontPickerCtrl*>(m_paramCtrls[i].ctrl);
                m_cfg->SetFont(m_paramCtrls[i].param, wxFontToStdFont(ctrl->GetSelectedFont()));
            }
            else if(m_paramCtrls[i].type == type_color)
            {
                wxColourPickerCtrl *ctrl = static_cast<wxColourPickerCtrl*>(m_paramCtrls[i].ctrl);
                m_cfg->SetColor(m_paramCtrls[i].param, wxColorToStdColor(ctrl->GetColour()));
            }
            else if(m_paramCtrls[i].type == type_choice_index)
            {
                wxChoice *ctrl = static_cast<wxChoice*>(m_paramCtrls[i].ctrl);
                m_cfg->SetInt(m_paramCtrls[i].param, ctrl->GetSelection());
            }
            else if(m_paramCtrls[i].type == type_color_font)
            {
                MyFontColorPicker *ctrl = static_cast<MyFontColorPicker*>(m_paramCtrls[i].ctrl);
                m_cfg->SetColorFont(m_paramCtrls[i].param, ctrl->GetStdColorFont());
            }
            else
            {
                std::string msg = "DialogProperty : unknown type \"";
                msg += nbToString(m_paramCtrls[i].type);
                msg += "\" !";
                throw ticpp::Exception(msg.c_str());
            }
        }
    }
}
//------------------------------------------------------------------------------
//----------------------------- MyNotebook -------------------------------------
//------------------------------------------------------------------------------
MyGearsPanel::MyGearsPanel(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, const wxString& name) : wxPanel(parent, id, pos, size, style, name)
{
    wxString ImgFileName = findFile(_T("data/gears.png"));
    if(wxFileExists(ImgFileName))
    {
        wxImage img(ImgFileName, wxBITMAP_TYPE_PNG);

        m_bmp = wxBitmap(img);
    }

    Connect( wxEVT_PAINT, wxPaintEventHandler(MyGearsPanel::OnPaint));
}
//------------------------------------------------------------------------------
void MyGearsPanel::OnPaint(wxPaintEvent& event)
{
    // DC
    wxPaintDC realDc(this);
    wxPaintDC *dc = &realDc; // Habitude

    // Background
    int x = GetSize().GetWidth() - m_bmp.GetWidth();
    int y = GetSize().GetHeight() - m_bmp.GetHeight();

    dc->DrawBitmap(m_bmp, x, y, true);

    // Liberartion du DC
    dc->SetBrush(wxNullBrush);
    dc->SetPen(wxNullPen);
}
