///File : MainFrame.hpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#if not defined MAIN_FRAME_HPP_DEVIS_MAKER
#define MAIN_FRAME_HPP_DEVIS_MAKER
//------------------------------------------------------------------------------
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif
//------------------------------------------------------------------------------
#include <wx/print.h>
#include <wx/html/helpctrl.h>
#include <wx/stdpaths.h>
#include <wx/dnd.h>
#include <wx/aui/auibook.h>
#include <wx/filedlg.h>
//------------------------------------------------------------------------------
#include <DevisManager.hpp>
//------------------------------------------------------------------------------
#include "strWxStdConv.hpp"
//------------------------------------------------------------------------------
#include "MainPanel.hpp"
#include "findFile.hpp"
#include "Printing.hpp"
#include "MainPanel.hpp"
#include "DialogAbout.hpp"
#include "DialogInfoDevis.hpp"
#include "DialogDevisPrices.hpp"
#include "DialogInfoGeneral.hpp"
#include "buildVersionOptions.hpp"

#if defined USE_MULTIPLE_CONFIG
    #include "DialogMultiCfgManager.hpp"
#endif
//------------------------------------------------------------------------------

class MainPanel;
class MainFrame;
//------------------------------------------------------------------------------
class MyFileDropTarget : public wxFileDropTarget
{
public:
    MyFileDropTarget(MainFrame *frame);

    virtual bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames);

private:
    MainFrame *m_frame;
};
//------------------------------------------------------------------------------
class MainFrame : public wxFrame
{
public:
    MainFrame(const wxString& title = _T(""), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE);
    ~MainFrame();
    void OnQuit(wxCommandEvent& event);
    void OnAbout(wxCommandEvent& event);
    void OnOpen(wxCommandEvent& event);
    void OnDevisProperties(wxCommandEvent& event);
    void OnDevisPrices(wxCommandEvent& event);
    void OnSave(wxCommandEvent& event);
    void OnSaveAs(wxCommandEvent& event);
    void OnHelp(wxCommandEvent& event);
    void OnClose(wxCommandEvent& event);
    void OnFrameClose(wxCloseEvent& event);
    void OnPrintPreview(wxCommandEvent& event);
    void OnPageSetup(wxCommandEvent& event);
    void OnPrint(wxCommandEvent& event);
    void OnNew(wxCommandEvent& event);
    void OnLunchDBEdit(wxCommandEvent& event);
    void OnRefresh(wxCommandEvent& event);
    void OnPreferences(wxCommandEvent& event);

    #if defined USE_MULTIPLE_FILE
    void OnNotebookChanged(wxAuiNotebookEvent& event);
    void OnNotebookClosing(wxAuiNotebookEvent& event);
    void OnNotebookClosed(wxAuiNotebookEvent& event);

    void OnSaveAll(wxCommandEvent& event);
    void OnCloseAll(wxCommandEvent& event);

    void RefreshNotebook();
    #endif

    #if defined USE_MULTIPLE_CONFIG
    void OnDialogMultiCfgManager(wxCommandEvent& event);
    void OnMenuItemsSelectCfg(wxCommandEvent& event);

    void FeedCfgListMenu();
    #endif

    void OnUndo(wxCommandEvent& event);
    void OnRedo(wxCommandEvent& event);

    void ReloadSectionItemDB();
    void ShowDevisCleanedMsg();
    void LoadFile(wxString selectedFile);
    bool AskToSave();

    // Menu
    wxMenuItem* CreateImageMenuItem( wxMenu *parentMenu, wxString dataFilename, const int &id = wxID_ANY, const wxString &value = _T("")) const;

    // Refresh
    void MyRefresh(const bool &withChild = true);
    void EnableMenus(bool enable);
    void RefreshUndoRedo();

    wxString GetBackupDescription(const int &pos);
protected:
    MainPanel *m_MainPanel;

    // File :
    wxMenuItem *m_MenuItemSave, *m_MenuItemSaveAs, *m_MenuItemClose, *m_MenuItemPrintPreview, *m_MenuItemPrint, *m_MenuItemPrintSetup;
    // Edit :
    wxMenuItem *m_MenuItemDevisProperties, *m_MenuItemDevisPrices;
    // Tools :
    #if defined USE_MULTIPLE_CONFIG
    wxMenuItem *m_MenuItemSaveAll, *m_MenuItemCloseAll;

    std::vector<wxMenuItem*> m_MenuItemsCfgs;
    wxMenu *m_cfgListMenu;
    #endif

    #if defined USE_MULTIPLE_FILE
    wxAuiNotebook *m_book;

    wxBitmap m_bitmapSaved, m_bitmapToSave;
    #endif

    // Undo / Redo
    wxString m_undoStr, m_undoKey;
    wxString m_redoStr, m_redoKey;

    wxMenuItem *m_ItemRedo, *m_ItemUndo, *m_ItemSelectAll;

    // Menu Bar
    wxMenuBar *m_MenuBar;

    // Printing
    wxPrintData m_printData;
    wxPrintDialogData m_printDialogData;
    wxPageSetupDialogData m_pageSetupDialogData;

    // DevisManager
    DevisManager *m_DevisManager;
};

#endif //MAIN_FRAME_HPP_DEVIS_MAKER
