///File : Image.hpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#ifndef MDO_SIZER_HPP_DEVIS_MAKER
#define MDO_SIZER_HPP_DEVIS_MAKER
//------------------------------------------------------------------------------
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif
//------------------------------------------------------------------------------
#include "config.hpp"
#include "buildVersionOptions.hpp"
//------------------------------------------------------------------------------
#include <algorithm>
//------------------------------------------------------------------------------
#include <DevisManager.hpp>
#include "DrawableObject.hpp"
#include "mdo.hpp"
//------------------------------------------------------------------------------
#include "strWxStdConv.hpp"
//------------------------------------------------------------------------------

namespace mdo
{
    #define MDO_SIZER_TYPE_NAME "mdoSizer"

	struct StructSizerData
	{
	    bool isVertical;
		bool isBorder;
        std::vector<DrawableObject*> mdoList;
        std::string mdoStrList;
	};

	class Sizer : public DrawableObject
	{
	public:
        Sizer(ConfigManager *cfg, ConfigManager *devisCfg, DrawableObject *parrentSizer, const std::string &name = "");

        // Cfg
		virtual void LoadFromCfg(const std::string &name = "");
		virtual void SaveToCfg(const std::string &name = "");
		virtual void CreateToCfg(const std::string &name = "");
		virtual void RemoveFromCfg(const std::string &name = "", const bool &withChilds = true);
		virtual void EditPropertyDialog(DialogProperty *window);

		virtual bool CalculSizeReal(wxDC *dc, const int &width, const int &askedHeight = 0);

		virtual void Draw(wxDC *dc, const wxPoint &pos);

        virtual ~Sizer();

        // Info
        bool IsVertical() const;
        bool IsRoot() const;

        // Special Sizer :
        DrawableObject* CreateMdo(const std::string &type);
        int GetMdoCount() const;
        DrawableObject* GetMdoPtr(const int &pos);

        // Changement
        void EraseMdo(DrawableObject *mdo);
        //void RenameMdo(DrawableObject *mdo, const std::string &newName);
        void ReceiveMdo(DrawableObject *mdo, const DrawableObject *after = NULL, const bool &addAfterDest = true);
        void CopyMdo(DrawableObject *mdo, const DrawableObject *after = NULL, const bool &addAfterDest = true);
        void UnlinkMdo(const DrawableObject *mdo); // Remove Mdo from list but do not destroy it
        int GetMdoPos(const DrawableObject *mdo) const;
        void RefreshParentLink();
        void CreateOwnCopy();

        // Special Tree Ctrl & Sizer
        bool SearchTreeItemId(const wxTreeItemId &treeItemId, DrawableObject *&mdo); // On prends des référence de pointeur pour modifier les pointeurs eux même.

	protected:
        DrawableObject* AddMdo(const std::string &type, const std::string &name);
        void CreateMdoListStr();
        DrawableObject* LoadMdo(const std::string &type, const std::string &name);
        void EraseMdo(const int &pos);
        void ClearMdos();


        StructSizerData m_data;
	};
}


#endif // MDO_SIZER_HPP_DEVIS_MAKER
