///File : Spacer.cpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "Spacer.hpp"
//------------------------------------------------------------------------------
namespace mdo
{

//------------------------------------------------------------------------------
//---------------------------------- Spacer -------------------------------------
//------------------------------------------------------------------------------
bool Spacer::CalculSizeReal(wxDC *dc, const int &width, const int &askedHeight)
{
    m_isReadyToDraw = true;

    m_sizeReal = wxSize(width, askedHeight);
}
//------------------------------------------------------------------------------
Spacer::Spacer(ConfigManager *cfg, ConfigManager *devisCfg, DrawableObject *parrentSizer, const std::string &name) : DrawableObject(cfg, devisCfg, parrentSizer, name)
{
    m_typeName = MDO_SPACER_TYPE_NAME;
    SetConfigName(""); // Rechargement du prefix
}
//------------------------------------------------------------------------------
void Spacer::LoadFromCfg(const std::string &name)
{
    DrawableObject::LoadFromCfg(name);
    m_data.isBorder = m_cfg->GetBool(m_cfgPrefix + "isBorder");
}
//------------------------------------------------------------------------------
void Spacer::SaveToCfg(const std::string &name)
{
    DrawableObject::SaveToCfg(name);
    m_cfg->SetBool(m_cfgPrefix + "isBorder", m_data.isBorder);
}
//------------------------------------------------------------------------------
void Spacer::CreateToCfg(const std::string &name)
{
    DrawableObject::CreateToCfg(name);
    m_cfg->SetBool(m_cfgPrefix + "isBorder", false);

    LoadFromCfg();
}
//------------------------------------------------------------------------------
void Spacer::RemoveFromCfg(const std::string &name, const bool &withChilds)
{
    DrawableObject::RemoveFromCfg(name);
    m_cfg->RemoveBool(m_cfgPrefix + "isBorder");
}
//------------------------------------------------------------------------------
void Spacer::EditPropertyDialog(DialogProperty *window)
{
    DrawableObject::EditPropertyDialog(window);

    wxPanel *PagePanel;
    wxBoxSizer *PageSizer;

    // Special Mdo
    PagePanel = new wxPanel(window->GetNotebook(), wxID_ANY);
    PageSizer = new wxBoxSizer(wxVERTICAL);

    window->AddParamCtrl(type_check, PagePanel, PageSizer, m_cfgPrefix + "isBorder", _("Add border"));

    PagePanel->SetSizer(PageSizer);
    window->GetNotebook()->AddPage(PagePanel, _("Object"));
}
//------------------------------------------------------------------------------
void Spacer::Draw(wxDC *dc, const wxPoint &pos)
{
    ThrowIfNotReadyToDraw();

    if(m_data.isBorder)
    {
        // Bordures
        dc->DrawRoundedRectangle(pos.x, pos.y, GetSizeReal().GetWidth(), GetSizeReal().GetHeight(), m_cfg->GetInt("arcSize"));
    }
}
//------------------------------------------------------------------------------


} // namespace mdo
